/******************************************************
   File name: DLLCode.h 

   This file contains MFC objects, hints, 
   you should use a MFC Client. 
   Notice: we use the same header file for compiling 
   the .DLL and the .exe (application).
   This header file defines a macro which export the
   target DLL objects if we are building
   a DLL, otherwise it import the DLL objects into an
   application which uses the DLL. If
   we define_EX (a preprocessor identifier), 
   then the preprocessor define macro
   (a mnemonic for: <B><I>DLL</I></B> import/export <B><I>Dir</I></B>ection)
   becomes an export instruction,
   otherwise its an import instruction by default. 
******************************************************/ 

	void CopyTextToClipboard(CString strData);
	CString Int2CString(int nNum, int nBase);

extern "C" { 

class ShellEx
{
	public:
		ShellEx();
		~ShellEx();
		CString PathToApp();
		void RunApp(CString szApp);
		void OpenFileDefaultApp(CString szFilePath);
		void OpenExploreFolder(CString szOperation, CString szPath, int nShowMode);
		CString PathFromFileDlg(BOOL bOpen);
		void PrintDocumentFile(CString szFilePath);
		typedef char StringType[MAX_PATH];
		CString GetFolder(CString szTitle);
};


class ColorRefConvert
{
	public:
		ColorRefConvert();
		~ColorRefConvert();
		CString ColorRef2RGB(COLORREF crVar);
		CString ColorRef2RGB_Red(COLORREF crVar);
		CString ColorRef2RGB_Green(COLORREF crVar);
		CString ColorRef2RGB_Blue(COLORREF crVar);
		CString ColorRef2Hex(COLORREF crVar);
		CString ConvertHex2RGB(CString szHex);
};
};

#ifndef _RIJNDAEL_H_
#define _RIJNDAEL_H_

//
// File : rijndael.h
// Creation date : Sun Nov 5 2000 03:21:05 CEST
// Author : Szymon Stefanek (stefanek@tin.it)
//
// Another implementation of the Rijndael cipher.
// This is intended to be an easily usable library file.
// This code is public domain.
// Based on the Vincent Rijmen and K.U.Leuven implementation 2.4.
//

//
// Original Copyright notice:
//
//    rijndael-alg-fst.c   v2.4   April '2000
//    rijndael-alg-fst.h
//    rijndael-api-fst.c
//    rijndael-api-fst.h
//
//    Optimised ANSI C code
//
//    authors: v1.0: Antoon Bosselaers
//             v2.0: Vincent Rijmen, K.U.Leuven
//             v2.3: Paulo Barreto
//             v2.4: Vincent Rijmen, K.U.Leuven
//
//    This code is placed in the public domain.
//

//
// This implementation works on 128 , 192 , 256 bit keys
// and on 128 bit blocks
//

//
// Example of usage:
//
//  // Input data
//  unsigned char key[32];                       // The key
//  initializeYour256BitKey();                   // Obviously initialized with sth
//  const unsigned char * plainText = getYourPlainText(); // Your plain text
//  int plainTextLen = strlen(plainText);        // Plain text length
//
//  // Encrypting
//  Rijndael rin;
//  unsigned char output[plainTextLen + 16];
//
//  rin.init(Rijndael::CBC,Rijndael::Encrypt,key,Rijndael::Key32Bytes);
//  // It is a good idea to check the error code
//  int len = rin.padEncrypt(plainText,len,output);
//  if(len >= 0)useYourEncryptedText();
//  else encryptError(len);
//
//  // Decrypting: we can reuse the same object
//  unsigned char output2[len];
//  rin.init(Rijndael::CBC,Rijndael::Decrypt,key,Rijndael::Key32Bytes));
//  len = rin.padDecrypt(output,len,output2);
//  if(len >= 0)useYourDecryptedText();
//  else decryptError(len);
//

#define _MAX_KEY_COLUMNS (256/32)
#define _MAX_ROUNDS      14
#define MAX_IV_SIZE      16

// We assume that unsigned int is 32 bits long.... 
typedef unsigned char  UINT8;
typedef unsigned int   UINT32;
typedef unsigned short UINT16;

// Error codes
#define RIJNDAEL_SUCCESS 0
#define RIJNDAEL_UNSUPPORTED_MODE -1
#define RIJNDAEL_UNSUPPORTED_DIRECTION -2
#define RIJNDAEL_UNSUPPORTED_KEY_LENGTH -3
#define RIJNDAEL_BAD_KEY -4
#define RIJNDAEL_NOT_INITIALIZED -5
#define RIJNDAEL_BAD_DIRECTION -6
#define RIJNDAEL_CORRUPTED_DATA -7

extern "C" { 

class Rijndael
{	
public:
	enum Direction { Encrypt , Decrypt };
	enum Mode { ECB , CBC , CFB1 };
	enum KeyLength { Key16Bytes , Key24Bytes , Key32Bytes };
	//
	// Creates a Rijndael cipher object
	// You have to call init() before you can encrypt or decrypt stuff
	//
	Rijndael();
	~Rijndael();
protected:
	// Internal stuff
	enum State { Valid , Invalid };

	State     m_state;
	Mode      m_mode;
	Direction m_direction;
	UINT8     m_initVector[MAX_IV_SIZE];
	UINT32    m_uRounds;
	UINT8     m_expandedKey[_MAX_ROUNDS+1][4][4];
public:
	//////////////////////////////////////////////////////////////////////////////////////////
	// API
	//////////////////////////////////////////////////////////////////////////////////////////

	// init(): Initializes the crypt session
	// Returns RIJNDAEL_SUCCESS or an error code
	// mode      : Rijndael::ECB, Rijndael::CBC or Rijndael::CFB1
	//             You have to use the same mode for encrypting and decrypting
	// dir       : Rijndael::Encrypt or Rijndael::Decrypt
	//             A cipher instance works only in one direction
	//             (Well , it could be easily modified to work in both
	//             directions with a single init() call, but it looks
	//             useless to me...anyway , it is a matter of generating
	//             two expanded keys)
	// key       : array of unsigned octets , it can be 16 , 24 or 32 bytes long
	//             this CAN be binary data (it is not expected to be null terminated)
	// keyLen    : Rijndael::Key16Bytes , Rijndael::Key24Bytes or Rijndael::Key32Bytes
	// initVector: initialization vector, you will usually use 0 here
	int init(Mode mode,Direction dir,const UINT8 *key,KeyLength keyLen,UINT8 * initVector = 0);


	// Encrypts the input array (can be binary data)
	// The input array length must be a multiple of 16 bytes, the remaining part
	// is DISCARDED.
	// so it actually encrypts inputLen / 128 blocks of input and puts it in outBuffer
	// Input len is in BITS!
	// outBuffer must be at least inputLen / 8 bytes long.
	// Returns the encrypted buffer length in BITS or an error code < 0 in case of error
	int blockEncrypt(const UINT8 *input, int inputLen, UINT8 *outBuffer);


	// Encrypts the input array (can be binary data)
	// The input array can be any length , it is automatically padded on a 16 byte boundary.
	// Input len is in BYTES!
	// outBuffer must be at least (inputLen + 16) bytes long
	// Returns the encrypted buffer length in BYTES or an error code < 0 in case of error
	int padEncrypt(const UINT8 *input, int inputOctets, UINT8 *outBuffer);
	
	
	// Decrypts the input vector
	// Input len is in BITS!
	// outBuffer must be at least inputLen / 8 bytes long
	// Returns the decrypted buffer length in BITS and an error code < 0 in case of error
	int blockDecrypt(const UINT8 *input, int inputLen, UINT8 *outBuffer);
	
	
	// Decrypts the input vector
	// Input len is in BYTES!
	// outBuffer must be at least inputLen bytes long
	// Returns the decrypted buffer length in BYTES and an error code < 0 in case of error
	int padDecrypt(const UINT8 *input, int inputOctets, UINT8 *outBuffer);


protected:
	void keySched(UINT8 key[_MAX_KEY_COLUMNS][4]);
	void keyEncToDec();
	void encrypt(const UINT8 a[16], UINT8 b[16]);
	void decrypt(const UINT8 a[16], UINT8 b[16]);
};
};
#endif // _RIJNDAEL_H_

// DCipher.h: interface for the DCipher class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_DCIPHER_H__85EBF52D_4509_43B1_937D_CD167C7056D9__INCLUDED_)
#define AFX_DCIPHER_H__85EBF52D_4509_43B1_937D_CD167C7056D9__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/*************************************************
Return values 
*************************************************/
#define _ReadBufSize 1000000
#define NO_ERRORS_DONE 0 //No errors.
#define ERROR_SRC_FILE 1 //Can't open the source file.
#define ERROR_DST_FILE 2 //Can't create the new file.
#define INSUFFICIENT_MEMORY 3 //ChunkSize it too large
#define READ_WRITE_ERROR 7 // error in encryption/decryption process
#define ALLOC_ERROR 8 //error allocating memory to buffers
#define BAD_SIGNATURE 9 //user signature doesn't match
/*************************************************/
#define ENCRYPTED_FILE 5 //The file is an encrypted file.
#define NORMAL_FILE 6 //The file is a normal file.
/*************************************************/
#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#include<malloc.h>
/*************************************************/


extern "C" { 

class DCipher{ //Declaring the class.


public: 

///////////////////////////////
// Constructors & Destructor
/////////////////////////////////
DCipher();

virtual ~DCipher();

////////////////////////////////////
// Implementation
////////////////////////////////////

int EncryptFile(const char *src, const char* pwd, int head[19]);
int DecryptFile(const char *src, const char* pwd, int head[19]);

char* GetSignature(const char *file);
void SetChunkSize(int numKB);

char* CalcMD5FromString(const char *s8_Input);
char* CalcMD5FromFile  (const char *s8_Path);


private: 

char SrcFile[256]; //The source file name.
char OutFile[256]; //The new file name.
long int FileSize; //The file size.
long int CurPosition; //The current position inside the file.
int header[26];  //19 user sig + 2 pad + 4 fileid bytes at top of file
char signature[20]; //signature text string returned by GetSignature()
char password[33]; //The encryption key 64 bytes max.
unsigned char key[33];	//the Rijndael key
long int liChunkSize;

int CheckIfEncrypted();
int MarkTheFile();
int GetFileLength();
int CheckMemSize();

    struct MD5Context 
    {
        unsigned long buf[4];
        unsigned long bits[2];
        unsigned char in[64];
    };

void MD5Init();
void MD5Update(unsigned char *buf, unsigned len);
void MD5Final (unsigned char digest[16]);
    
void MD5Transform(unsigned long buf[4], unsigned long in[16]);
char* MD5FinalToString();
char* MD5ToEncryptionKey();
void FreeBuffer();
void byteReverse (unsigned char *buf, unsigned longs);

char *mp_s8ReadBuffer;
MD5Context ctx;
char   ms8_MD5[40]; // Output buffer
char pkey[40];
};
};
#endif

